@@ -26,8 +26,8 @@ from pyGDM2 import linear as linear_py
 
 
 __name__ = 'pyGDM2'
-__version__ = '1.1.1.1'
-__date__ = "06/04/2021"
+__version__ = '1.1.2'
+__date__ = "12/10/2021"   # MM/DD/YYY
 __author__ = 'Peter R. Wiecha'
 
 __all__ = ["core", "materials", "structures", "fields", 
