@@ -121,7 +121,7 @@ class simulation(object):
     __name__ = "pygdm2 simulation"
     
     
-    def __init__(self, struct, efield, dyads=None, dtype='f'):
+    def __init__(self, struct, efield, dyads=None, dtype='f', verbose=True):
         """Initialization"""
         
         ## --- struct includes: geometry, material(s), environment
@@ -390,8 +390,10 @@ def scatter(sim, method='lu', calc_H=False, verbose=True, callback=None, **kwarg
                                   )
             if not cb_continue:
                 return -1
-            
-    gc.collect()
+        
+        ## free RAM for next iteration
+        del K
+        gc.collect()
     
     return 1
 
