@@ -328,7 +328,7 @@ def _three_layer_pw(wavelength, theta_inc, polar, z_d, spacing, n1, n2, n3, x, y
                 Ez = n3*(cphase3*k3[0]/(eps3*k0)+cmagp3*cphase3p*k3p[0]/(eps3*k0))
                 
                 Bx = 0.
-                By = -cphase3-cmagp3*cphase3p
+                By = n3*(-cphase3-cmagp3*cphase3p)
                 Bz = 0.
                 
         elif z_d<z<z_u:
@@ -350,7 +350,7 @@ def _three_layer_pw(wavelength, theta_inc, polar, z_d, spacing, n1, n2, n3, x, y
                 Ez = n3*(cmag2*cphase2*k2[0]/(eps2*k0)+cmagp2*cphase2p*k2p[0]/(eps2*k0))
                 
                 Bx = 0.
-                By = -cmag2*cphase2-cmagp2*cphase2p
+                By = n3*(-cmag2*cphase2-cmagp2*cphase2p)
                 Bz = 0.
                 
         else:
@@ -372,7 +372,7 @@ def _three_layer_pw(wavelength, theta_inc, polar, z_d, spacing, n1, n2, n3, x, y
                 Ez = n3*(cmag1*cphase1*k1[0]/(eps1*k0))
                 
                 Bx = 0.
-                By = -cmag1*cphase1
+                By = n3*(-cmag1*cphase1)
                 Bz = 0.
 
     else :    
@@ -455,7 +455,7 @@ def _three_layer_pw(wavelength, theta_inc, polar, z_d, spacing, n1, n2, n3, x, y
                 Ez = n1*(cphase1*k1[0]/(eps1*k0)+cmagp1*cphase1p*k1p[0]/(eps1*k0))
                 
                 Bx = 0.
-                By = -cphase1-cmagp1*cphase1p
+                By = n1*(-cphase1-cmagp1*cphase1p)
                 Bz = 0.
                 
         elif z_d<z<z_u:
@@ -477,7 +477,7 @@ def _three_layer_pw(wavelength, theta_inc, polar, z_d, spacing, n1, n2, n3, x, y
                 Ez = n1*(cmag2*cphase2*k2[0]/(eps2*k0)+cmagp2*cphase2p*k2p[0]/(eps2*k0))
                 
                 Bx = 0.
-                By = -cmag2*cphase2-cmagp2*cphase2p
+                By = n1*(-cmag2*cphase2-cmagp2*cphase2p)
                 Bz = 0.
                 
                 
@@ -500,7 +500,7 @@ def _three_layer_pw(wavelength, theta_inc, polar, z_d, spacing, n1, n2, n3, x, y
                 Ez = n1*(cmag3*cphase3*k3[0]/(eps3*k0))
                 
                 Bx = 0.
-                By = -cmag3*cphase3
+                By = n1*(-cmag3*cphase3)
                 Bz = 0.
     
     return E0*Ex, E0*Ey, E0*Ez, E0*Bx, E0*By, E0*Bz
@@ -608,6 +608,14 @@ def plane_wave(pos, env_dict, wavelength,
         E_p = 1.0 * np.cos(theta * np.pi/180.)
         E_s = 1.0 * np.sin(theta * np.pi/180.)
     
+    ## normalize incident amplitude by refractive index
+    # if abs(inc_angle)<90 and np.abs(cn1)!=1: # inc from zone n1
+    #     E_s = E_s / cn1**0.5
+    #     E_p = E_p / cn1**0.5
+    # elif abs(inc_angle)>=90 and np.abs(cn3)!=1: # inc from zone n3
+    #     E_s = E_s / cn3**0.5
+    #     E_p = E_p / cn3**0.5
+    
     for i,R in enumerate(pos):
         if inc_plane.lower() in ['yz', 'zy']:
             y,x,z = R
@@ -636,9 +644,10 @@ def plane_wave(pos, env_dict, wavelength,
         
         ## optional scattering plane modification
         if inc_plane.lower() in ['yz', 'zy']:
-            ex_s,ey_s,ez_s = ey_s,ex_s,ez_s
+            ex_s,ey_s,ez_s = -ey_s,-ex_s,-ez_s
             bx_s,by_s,bz_s = by_s,bx_s,bz_s
-            ex_p,ey_p,ez_p = ey_p,ex_p,ez_p
+            
+            ex_p,ey_p,ez_p = -ey_p,-ex_p,-ez_p
             bx_p,by_p,bz_p = by_p,bx_p,bz_p
         
         if returnField.lower() == 'e':
