@@ -63,13 +63,13 @@ def _calc_repropagation(P, Escat, G_dyad_list):
 
 
 
-#==============================================================================
-# Linear Field processing functions
-#==============================================================================
-#import numba
-#@numba.njit(parallel=True)
+
+
+# =============================================================================
+# farfield functions
+# =============================================================================
 def _calc_extinct(wavelength, n_env, alpha_tensor, E0, E, 
-                  with_radiation_correction=False, normalization_E0=False):
+                  with_radiation_correction=False, normalization_E0=True):
     """actual calculation of extinction and absorption cross sections
     
     numba compatible. tested.
@@ -95,7 +95,7 @@ def _calc_extinct(wavelength, n_env, alpha_tensor, E0, E,
     return cext, csca, cabs
     
 
-def extinct(sim, field_index, with_radiation_correction=False, normalization_E0=False):
+def extinct(sim, field_index, with_radiation_correction=False, normalization_E0=True):
     """Extinction, scattering and absorption cross-sections
     
     Calculates extinction, scattering and absorption crosssections
@@ -121,7 +121,7 @@ def extinct(sim, field_index, with_radiation_correction=False, normalization_E0=
         discretization stepsizes, but has only a weak influence on simulations
         with fine meshes.
         
-    normalization_E0 : bool, default: False
+    normalization_E0 : bool, default: True
         normalizes sections to peak of incident field intensity inside structure
     
     Returns
@@ -165,308 +165,9 @@ def extinct(sim, field_index, with_radiation_correction=False, normalization_E0=
     
     return _calc_extinct(wavelength, n_env, alpha_tensor, E0, E, 
                          with_radiation_correction, normalization_E0)
-
-
-
-
-
-def multipole_decomp(sim, field_index, r0=None, quadrupoles=False):
-    """multipole decomposition of nanostructure optical response
-    
-    ** ------- FUNCTION STILL UNDER TESTING ------- **
-    
-    Multipole decomposition of electromagnetic field inside nanostructure for 
-    electric and magnetic dipole and quadrupole moments.
-
-
-    Parameters
-    ----------
-    sim : :class:`.core.simulation`
-        simulation description
-    
-    field_index : int
-        index of evaluated self-consistent field to use for calculation. Can be
-        obtained for specific parameter-set using :func:`.tools.get_closest_field_index`
-    
-    r0 : array, default: None
-        [x,y,z] position of multipole decomposition development. 
-        If `None`, use structure's center of gravity
-        
-    quadrupoles : bool, default: False
-        if True, return also quadrupole moments. Otherwise return only 
-        electric and magnetic dipole moments.
-        This is a direct implementation of  Evlyukhin et al (see Notes) and is not tested.
-    
-    
-    Returns
-    -------
-    
-    p : 3-vector
-        electric dipole moment
-    
-    m : 3-vector
-        magnetic dipole moment
-        
-    q : 3x3 tensor
-        electric quadrupole moment
-        
-    mq : 3x3 tensor
-        magnetic quadrupole moment
-    
-    
-    Notes
-    -----
-    For details about the method, see: 
-    Evlyukhin, A. B. et al. *Multipole analysis of light scattering by 
-    arbitrary-shaped nanoparticles on a plane surface.*, 
-    JOSA B 30, 2589 (2013)
-    """
-    if sim.E is None: 
-        raise ValueError("Error: Scattered field inside the structure not yet evaluated. Run `core.scatter` simulation first.")
-    
-    warnings.warn("Multipole decomposition is a new functionality still under testing. " + 
-                  "Please use with caution.")
-    
-    field_params = sim.E[field_index][0]
-    wavelength = field_params['wavelength']
-    k0 = 2*np.pi / wavelength
-    
-    geo = sim.struct.geometry
-    E = sim.E[field_index][1]
-    ex, ey, ez = E.T
-    
-    ## --- polarizability of each meshpoint
-    sim.struct.setWavelength(wavelength)
-    alpha_tensor = sim.dyads.getPolarizabilityTensor(wavelength, sim.struct)
-    
-    
-    ## --- use center of gravity for multimode expansion
-    if r0 is None:
-        r0 = np.average(geo, axis=0)
-    Dr = geo - r0
-    
-    
-    ## --- total electric dipole moment:
-    ##     P = sum_i p(r_i) = chi V_cell sum_i E(r_i)
-    P = np.matmul(alpha_tensor, E[...,None])[...,0]
-    p = np.sum(P, axis=0)  # = volume integral
-    
-    ## --- total magnetic dipole moment: (X=curl)
-    ##     M = -iw0/2c  sum_i (r_i - r0) X p(r_i) 
-    ##       = -ik0/2 sum_i chi Vcell (r_i - r0) X E(r_i)
-    M = -(1j*k0/2.) * np.cross(Dr, P)
-    m = np.sum(M, axis=0)
-        
-    if quadrupoles:
-        ## --- total electric quadrupole moment: (X=curl)
-        ##     Q = 3 * sum_i (Dr p(r_i) + p(r_i) Dr)
-        ##       = 3 * sum_i chi Vcell (Dr E(r_i) + E(r_i) Dr)
-        Q = 3 * (np.einsum('li,lj->lji', Dr, P) + np.einsum('li,lj->lji', P, Dr))
-        q = np.sum(Q, axis=0)
-        
-        ## --- total magnetic dipole moment: (X=curl)
-        ##     M = -2iw0/3c  sum_i (r_i - r0) X p(r_i) 
-        ##       = -2ik0/3 sum_i chi Vcell (r_i - r0) X E(r_i)
-        MQ = -(2j*k0/3.) * np.einsum('li,lj->lji', np.cross(Dr, P), Dr)
-        mq = np.sum(MQ, axis=0)
-        
-        return p, m, q, mq
-
-    else:
-        return p, m
-        
-    
-def mutlipole_decomp(sim, field_index, r0=None):
-    """wrapper for correctly spelled function"""
-    warnings.warn('Please adapt your code to use the correctly spelled "multipole_decomp" function.')
-    return multipole_decomp(sim, field_index, r0, quadrupoles=True)
-
-    
-    
-def multipole_decomp_extinct(sim, field_index, r0=None, eps_dd=0.1, quadrupoles=False):
-    """extinction from multipole decomposition
-    
-    ** ------- FUNCTION STILL UNDER TESTING ------- **
-    
-    Returns extinction cross sections for electric and magnetic dipole and 
-    quadrupole moments of the multipole decomposition.
-    
-    
-    Parameters
-    ----------
-    sim : :class:`.core.simulation`
-        simulation description
-    
-    field_index : int
-        index of evaluated self-consistent field to use for calculation. Can be
-        obtained for specific parameter-set using :func:`.tools.get_closest_field_index`
-    
-    r0 : array, default: None
-        [x,y,z] position of mulipole decomposition development. 
-        If `None`, use structure's center of gravity
-    
-    
-    eps_dd : float, default: 0.1
-        numerical integration stepsize (in nm). Used for e/m quadrupole.
-    
-    quadrupoles : bool, default: False
-        if True, return also quadrupole moments. Otherwise return only 
-        electric and magnetic dipole moments.
-        This is a direct implementation of  Evlyukhin et al (see Notes) and is not tested.
-    
-    Returns
-    -------
-    
-    sigma_ext_p : float
-        electric dipole extinction cross section (in nm^2)
-    
-    sigma_ext_m : float
-        magnetic dipole extinction cross section (in nm^2)
-        
-    sigma_ext_q : float
-        electric quadrupole extinction cross section (in nm^2)
-        
-    sigma_ext_mq : float
-        magnetic quadrupole extinction cross section (in nm^2)
-    
-    
-    Notes
-    -----
-    For details about the method, see: 
-    Evlyukhin, A. B. et al. *Multipole analysis of light scattering by 
-    arbitrary-shaped nanoparticles on a plane surface.*, 
-    JOSA B 30, 2589 (2013)
-    
-    """
-    if sim.E is None: 
-        raise ValueError("Error: Scattered field inside the structure not yet evaluated. Run `core.scatter` simulation first.")
-    
-    ## --- by default, use center of gravity for multimode expansion
-    field_params = sim.E[field_index][0]
-    wavelength = field_params['wavelength']
-    sim.struct.setWavelength(wavelength)
-    k0 = 2*np.pi / wavelength
-    
-    geo = sim.struct.geometry
-    n_env = sim.dyads.getEnvironmentIndices(wavelength, geo[:1])[0]  # assume structure is fully in one environment
-    E = sim.E[field_index][1]
-    ex, ey, ez = E.T
-    
-    if r0 is None:
-        r0 = np.average(geo, axis=0)
-    
-    if quadrupoles:
-        p, m, q, mq = multipole_decomp(sim, field_index, r0=r0, quadrupoles=quadrupoles)
-    else:
-        p, m = multipole_decomp(sim, field_index, r0=r0, quadrupoles=quadrupoles)
-    
-# =============================================================================
-#     dipole extinction cross sections
-# =============================================================================
-    env_dict = sim.dyads.getConfigDictG(wavelength, sim.struct, sim.efield)
-    E0 = sim.efield.field_generator(r0[None,:], env_dict, **field_params)
-    H0 = sim.efield.field_generator(r0[None,:], env_dict, returnField='H', **field_params)
-    
-    E2in = np.sum(np.abs(E0)**2)     # incident intensity
-    sigma_ext_p = (4 * np.pi * k0 / n_env * 1/E2in * np.sum(np.conjugate(E0)*p)).imag
-    sigma_ext_m = (4 * np.pi * k0 / n_env * 1/E2in * np.sum(np.conjugate(H0)*m)).imag
-    
-# =============================================================================
-#     quadrupole extinction cross sections
-# =============================================================================
-    if quadrupoles:
-        DX_pos = np.array([r0-np.array([eps_dd,0,0]), r0+np.array([eps_dd,0,0])])
-        DY_pos = np.array([r0-np.array([0,eps_dd,0]), r0+np.array([0,eps_dd,0])])
-        DZ_pos = np.array([r0-np.array([0,0,eps_dd]), r0+np.array([0,0,eps_dd])])
-        EDXcj = np.conjugate(sim.efield.field_generator(DX_pos, env_dict, **field_params))
-        EDYcj = np.conjugate(sim.efield.field_generator(DY_pos, env_dict, **field_params))
-        EDZcj = np.conjugate(sim.efield.field_generator(DZ_pos, env_dict, **field_params))
-        gradE0cj = np.array([[EDXcj[1,0]-EDXcj[0,0], EDYcj[1,0]-EDYcj[0,0], EDZcj[1,0]-EDZcj[0,0]],
-                             [EDXcj[1,1]-EDXcj[1,1], EDYcj[1,1]-EDYcj[0,1], EDZcj[1,1]-EDZcj[0,1]],
-                             [EDXcj[1,2]-EDXcj[1,2], EDYcj[1,2]-EDYcj[0,2], EDZcj[1,2]-EDZcj[0,2]]])
-        sigma_ext_q = (np.pi * k0 / n_env * 1/(3. * E2in) * 
-                               np.sum(np.tensordot(gradE0cj + gradE0cj.T, q) )).imag
-        
-        HDXcj = np.conjugate(sim.efield.field_generator(DX_pos, env_dict, returnField='H', **field_params))
-        HDYcj = np.conjugate(sim.efield.field_generator(DY_pos, env_dict, returnField='H', **field_params))
-        HDZcj = np.conjugate(sim.efield.field_generator(DZ_pos, env_dict, returnField='H', **field_params))
-        gradH0cj = np.array([[HDXcj[1,0]-HDXcj[0,0], HDYcj[1,0]-HDYcj[0,0], HDZcj[1,0]-HDZcj[0,0]],
-                             [HDXcj[1,1]-HDXcj[1,1], HDYcj[1,1]-HDYcj[0,1], HDZcj[1,1]-HDZcj[0,1]],
-                             [HDXcj[1,2]-HDXcj[1,2], HDYcj[1,2]-HDYcj[0,2], HDZcj[1,2]-HDZcj[0,2]]])
-        sigma_ext_mq = (2 * np.pi * k0 / n_env * 1/E2in * 
-                               np.sum(np.tensordot(gradH0cj.T, mq) )).imag
-        
-        return sigma_ext_p, sigma_ext_m, sigma_ext_q, sigma_ext_mq
-
-    else:
-        return sigma_ext_p, sigma_ext_m
-
-def mutlipole_decomp_extinct(sim, field_index, r0=None, eps_dd=0.1):
-    """wrapper for correctly spelled function"""
-    warnings.warn('Please adapt your code to use the correctly spelled "multipole_decomp_extinct" function.')
-    return multipole_decomp_extinct(sim, field_index, r0, eps_dd, quadrupoles=True)
-
-
-
-# =============================================================================
-# internal field intensity helper
-# =============================================================================
-def internal_field_intensity(sim, field_index, which_value='average',
-                             which_field='E'):
-    """return internal field intensity
-    
-    returns either average or peak (min or max) internal field intensity for 
-    either electric of magnetic field.
-    
-    Parameters
-    ----------
-    sim : :class:`.core.simulation`
-        simulation description
-        
-    field_index : int
-        index of evaluated self-consistent field to use for calculation. Can be
-        obtained for specific parameter-set using :func:`.tools.get_closest_field_index`
-    
-    which_value : str, default: 'average'
-        - if 'average', 'avg' or 'mean': average field intensity
-        - if 'max' or 'maximum': maximum field intensity
-        - if 'min' or 'minimum': minimum field intensity
-    
-    which_field : str, default: 'E'
-        one of 'E', 'H' or 'both' (either electric or magnetic field or both)
-    
-    """
-    if sim.E is None: 
-        raise ValueError("Error: Scattered field inside the structure not yet evaluated. Run `core.scatter` simulation first.")
-    
-    ## --- E field
-    if which_field.lower() in ['e', 'both']:
-        if sim.E is not None:
-            E = sim.E[field_index][1]
-            I_E_mean = np.sum(np.abs(E)**2, axis=1).mean()
-        else:
-            raise Exception("E field not evaluated. Please run 'sim.scatter()'.")
-    
-    ## --- H field
-    if which_field.lower() in ['h', 'b', 'both']:
-        if sim.H is not None:
-            H = sim.H[field_index][1]
-            I_H_mean = np.sum(np.abs(H)**2, axis=1).mean()
-        else:
-            raise Exception("H field not evaluated. Please run 'sim.scatter(calc_H=True)'.")
-    
-    if which_field.lower() in ['h', 'b']:
-        return I_H_mean
-    elif which_field.lower() in ['e']:
-        return I_E_mean
-    else:
-        return I_E_mean, I_H_mean
     
 
 
-# =============================================================================
-# farfield
-# =============================================================================
 def farfield(sim, field_index, 
                 r_probe=None,
                 r=100000., 
@@ -500,7 +201,7 @@ def farfield(sim, field_index,
         (list of) coordinate(s) to evaluate farfield on. 
         Format: tuple (x,y,z) or list of 3 lists: [Xmap, Ymap, Zmap] 
         
-    r : float, default: 10000.
+    r : float, default: 100000.
         radius of integration sphere (distance to coordinate origin in nm)
         
     tetamin, tetamax : float, float; defaults: 0, np.pi/2
@@ -727,7 +428,9 @@ def farfield(sim, field_index,
 
 
 
-
+# =============================================================================
+# nearfield functions
+# =============================================================================
 def nearfield(sim, field_index, r_probe, which_fields=["Es","Et","Bs","Bt"],
                  val_inside_struct="field", N_neighbors_internal_field=4,
                  method=''):
@@ -960,10 +663,10 @@ def nearfield(sim, field_index, r_probe, which_fields=["Es","Et","Bs","Bt"],
         if f_type.lower() == "es":
             Escat = np.concatenate([r_probe, Escat], axis=1)
             return_field_list.append(Escat)
-        if f_type.lower() == "et" in which_fields:
+        if f_type.lower() == "et":
             Etot = np.concatenate([r_probe, Etot], axis=1)
             return_field_list.append(Etot)
-        if f_type.lower() == "e0" in which_fields:
+        if f_type.lower() == "e0":
             E0 = np.concatenate([r_probe, E0], axis=1)
             return_field_list.append(E0)
             
@@ -980,6 +683,105 @@ def nearfield(sim, field_index, r_probe, which_fields=["Es","Et","Bs","Bt"],
     return return_field_list
 
 
+
+def internal_field_intensity(sim, field_index, which_value='average',
+                             which_field='E'):
+    """return internal field intensity
+    
+    returns either average or peak (min or max) internal field intensity for 
+    either electric of magnetic field.
+    
+    Parameters
+    ----------
+    sim : :class:`.core.simulation`
+        simulation description
+        
+    field_index : int
+        index of evaluated self-consistent field to use for calculation. Can be
+        obtained for specific parameter-set using :func:`.tools.get_closest_field_index`
+    
+    which_value : str, default: 'average'
+        - if 'average', 'avg' or 'mean': average field intensity
+        - if 'max' or 'maximum': maximum field intensity
+        - if 'min' or 'minimum': minimum field intensity
+    
+    which_field : str, default: 'E'
+        one of 'E', 'H' or 'both' (either electric or magnetic field or both)
+    
+    """
+    if sim.E is None: 
+        raise ValueError("Error: Scattered field inside the structure not yet evaluated. Run `core.scatter` simulation first.")
+    
+    ## --- E field
+    if which_field.lower() in ['e', 'both']:
+        if sim.E is not None:
+            E = sim.E[field_index][1]
+            I_E_mean = np.sum(np.abs(E)**2, axis=1).mean()
+        else:
+            raise Exception("E field not evaluated. Please run 'sim.scatter()'.")
+    
+    ## --- H field
+    if which_field.lower() in ['h', 'b', 'both']:
+        if sim.H is not None:
+            H = sim.H[field_index][1]
+            I_H_mean = np.sum(np.abs(H)**2, axis=1).mean()
+        else:
+            raise Exception("H field not evaluated. Please run 'sim.scatter(calc_H=True)'.")
+    
+    if which_field.lower() in ['h', 'b']:
+        return I_H_mean
+    elif which_field.lower() in ['e']:
+        return I_E_mean
+    else:
+        return I_E_mean, I_H_mean
+
+
+
+def poynting(sim, field_index, r_probe, which_fields="tot"):
+    """calculate the time average Poynting vector
+    
+    calculate the time average Poynting vector of the scattered, total or 
+    fundamental (incident) field of a pyGDM simulation at one or more locations
+    
+    Parameters
+    ----------
+    sim : :class:`.core.simulation`
+        simulation description
+    
+    field_index : int
+        index of evaluated self-consistent field to use for calculation. Can be
+        obtained for specific parameter-set using :func:`.tools.get_closest_field_index`
+    
+    r_probe : tuple (x,y,z) or list of 3-lists/-tuples
+        (list of) coordinate(s) to evaluate nearfield on. 
+        Format: tuple (x,y,z) or list of 3 lists: [Xmap, Ymap, Zmap] 
+        (the latter can be generated e.g. using :func:`.tools.generate_NF_map`)
+    
+    which_fields : str, default: "tot"
+        which type of fields to use. one of: "scat", "tot", "0".
+        scattered field, total field (=E_scat + E_0) or incident field alone.
+        
+        
+    Returns
+    -------
+    P : list of 6-tuples (x,y,z, Px, Py, Pz)
+    """
+    
+    if "scat" == which_fields.lower():
+        wf= ['es', 'hs']
+    if "tot" == which_fields.lower():
+        wf= ['et', 'ht']
+    if "0" == which_fields.lower() or "in" == which_fields.lower():
+        wf= ['e0', 'h0']
+    
+    E, H = nearfield(sim, field_index, r_probe, which_fields=wf)
+    
+    P = np.copy(E)
+    P[:,3:] = np.cross(np.conj(E[:,3:]), H[:,3:])
+
+    return P.real
+
+
 # =============================================================================
 # field gradients
 # =============================================================================
@@ -1557,20 +1359,6 @@ def temperature(sim, field_index, r_probe,
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 
 
 
@@ -1579,124 +1367,57 @@ def temperature(sim, field_index, r_probe,
 
 
 # =============================================================================
-# experimental CUDA propagator functions
+# deprecations - to be removed in a future version
 # =============================================================================
-##
-## ---  Call like :
-##
-#     elif method.lower()=='cuda':
-#         G_NF = _calc_multidipole_NF_EE_GPU(sim.struct.geometry, r_probe, 
-#                                            wavelength, eps1, eps2, eps3, spacing)
-#         Escat = _calc_repropagation_GPU(P, G_NF)
+def multipole_decomp(sim, field_index, r0=None, quadrupoles=False):
+    """multipole decomposition of nanostructure optical response
+    
+    ** ------- DEPRECATED ------- **
+    
+    please use :func:`pyGDM2.multipole.multipole_decomposition_exact`
+    
+    """
+    from pyGDM2 import multipole
+    
+    warnings.warn("This function is deprecated and will be removed in a future version. " +
+                  "Please use the `pyGDM2.multipole` module instead.")
+    
+    if quadrupoles:
+        p, m, q, mq = multipole.multipole_decomposition_exact(
+                                    sim, field_index, r0, epsilon=0.01, 
+                                    which_moments=['p', 'm', 'qe', 'qm'])
+        return p, m, q, mq
 
-#     elif method.lower()=='cuda':
-#         G_NF = _calc_multidipole_NF_HE_GPU(sim.struct.geometry, r_probe, 
-#                                            wavelength, eps_env)
-#         Bscat = _calc_repropagation_GPU(P, G_NF)
-
-# def _calc_multidipole_NF_EE_GPU(dp_pos, r_probe, wavelength, eps1, eps2, eps3, spacing,
-#                                 threadsperblock=(16,16)):
-#     from numba import cuda
-    
-#     ## --- cuda version of propagator
-#     G_cuda = cuda.jit(device=True)(G_EE)
-    
-#     ## --- cuda kernel
-#     @cuda.jit()
-#     def multidp_kernel_cuda(dp_pos, r_probe, wavelength, eps1, eps2, eps3, spacing, M):
-#         """
-#         Code for kernel.
-#         """
-#         i, j = cuda.grid(2)    # indices of current cuda thread
-#         if i < M.shape[0] and j < M.shape[1]:
-#             _pos = dp_pos[i]
-#             _r = r_probe[j]
-#             xx, yy, zz, xy, xz, yx, yz, zx, zy = G_cuda(_pos, _r,
-#                                                wavelength, eps1, eps2, eps3, spacing)
-#             ## return list of Greens tensors
-#             M[i,j,0,0], M[i,j,1,1], M[i,j,2,2] = xx, yy, zz
-#             M[i,j,1,0], M[i,j,2,0], M[i,j,0,1] = yx, zx, xy
-#             M[i,j,2,1], M[i,j,0,2], M[i,j,1,2] = zy, xz, yz
-            
-#     ## --- cuda threads config
-#     M = np.zeros((len(dp_pos), len(r_probe),3,3), dtype=np.complex64)
-#     blockspergrid_x = math.ceil(M.shape[0] / threadsperblock[0])
-#     blockspergrid_y = math.ceil(M.shape[1] / threadsperblock[1])
-#     blockspergrid = (blockspergrid_x, blockspergrid_y)
-    
-#     ## -- run cuda kernel
-#     cuda.select_device(0)
-#     multidp_kernel_cuda[blockspergrid, threadsperblock](dp_pos, r_probe, wavelength, 
-#                                            eps1, eps2, eps3, spacing, M)
-#     cuda.close()
-    
-#     return M
+    else:
+        p, m = multipole.multipole_decomposition_exact(
+                                    sim, field_index, r0, epsilon=0.01, 
+                                    which_moments=['p', 'm'])
+        return p, m
 
 
-# def _calc_multidipole_NF_HE_GPU(dp_pos, r_probe, wavelength, eps_env, threadsperblock=(16,16)):
-#     from numba import cuda
-
-#     ## --- cuda version of propagator
-#     G_cuda = cuda.jit(device=True)(G0_HE)
+def multipole_decomp_extinct(sim, field_index, r0=None, eps_dd=0.1, quadrupoles=False):
+    """extinction from multipole decomposition
     
-#     ## --- cuda kernel
-#     @cuda.jit()
-#     def multidp_kernel_cuda(dp_pos, r_probe, wavelength, eps_env, M):
-#         """
-#         Code for kernel.
-#         """
-#         i, j = cuda.grid(2)    # indices of current cuda thread
-#         if i < M.shape[0] and j < M.shape[1]:
-#             _pos = dp_pos[i]
-#             _r = r_probe[j]
-#             xx, yy, zz, xy, xz, yx, yz, zx, zy = G_cuda(_pos, _r, wavelength, eps_env)
-#             ## return list of Greens tensors
-#             M[i,j,0,0], M[i,j,1,1], M[i,j,2,2] = xx, yy, zz
-#             M[i,j,1,0], M[i,j,2,0], M[i,j,0,1] = yx, zx, xy
-#             M[i,j,2,1], M[i,j,0,2], M[i,j,1,2] = zy, xz, yz
-            
-#     ## --- cuda threads config
-#     M = np.zeros((len(dp_pos), len(r_probe),3,3), dtype=np.complex64)
-#     blockspergrid_x = math.ceil(M.shape[0] / threadsperblock[0])
-#     blockspergrid_y = math.ceil(M.shape[1] / threadsperblock[1])
-#     blockspergrid = (blockspergrid_x, blockspergrid_y)
+    ** ------- DEPRECATED ------- **
     
-#     ## -- run cuda kernel
-#     cuda.select_device(0)
-#     multidp_kernel_cuda[blockspergrid, threadsperblock](dp_pos, r_probe, wavelength, eps_env, M)
-#     cuda.close()
+    please use :func:`pyGDM2.multipole.multipole_decomposition_exact`
+    """
+    from pyGDM2 import multipole
     
-#     return M
+    warnings.warn("This function is deprecated and will be removed in a future version. " +
+                  "Please use the `pyGDM2.multipole` module instead.")
+    
+    sigma_ext_p, sigma_ext_m, sigma_ext_q, sigma_ext_mq = multipole.extinct(
+                                                sim, field_index, r0, eps_dd)
+    if quadrupoles:
+        return sigma_ext_p, sigma_ext_m, sigma_ext_q, sigma_ext_mq
+    else:
+        return sigma_ext_p, sigma_ext_m
 
 
-# def _calc_repropagation_GPU(P, G_dyad_list, threadsperblock=(16,16)):
-#     from numba import cuda
+def mutlipole_decomp(sim, field_index, r0=None):
+    return multipole_decomp(sim, field_index, r0, quadrupoles=True)
+
+def mutlipole_decomp_extinct(sim, field_index, r0=None, eps_dd=0.1):
+    return multipole_decomp_extinct(sim, field_index, r0, eps_dd, quadrupoles=True)
 
-#     ## --- cuda kernel
-#     @cuda.jit()
-#     def repropa_kernel_cuda(P, Escat_all, G_dyad_list):
-#         """
-#         Code for kernel.
-#         """
-#         i, j = cuda.grid(2)    # indices of current cuda thread
-#         if i < G_dyad_list.shape[0] and j < G_dyad_list.shape[1]:
-#             _G = G_dyad_list[i, j]
-#             _P = P[i]
-# #            _Es = np.dot(_G, P[i])
-#             Escat_all[i, j, 0] = _G[0,0]*_P[0] + _G[0,1]*_P[1] + _G[0,2]*_P[2]
-#             Escat_all[i, j, 1] = _G[1,0]*_P[0] + _G[1,1]*_P[1] + _G[1,2]*_P[2]
-#             Escat_all[i, j, 2] = _G[2,0]*_P[0] + _G[2,1]*_P[1] + _G[2,2]*_P[2]
-    
-#     ## --- cuda threads config
-#     Escat_all = np.zeros(shape=(len(P), G_dyad_list.shape[1], 3), dtype=np.complex64)
-#     blockspergrid_x = math.ceil(Escat_all.shape[0] / threadsperblock[0])
-#     blockspergrid_y = math.ceil(Escat_all.shape[1] / threadsperblock[1])
-#     blockspergrid = (blockspergrid_x, blockspergrid_y)
-    
-#     ## -- run cuda kernel
-#     cuda.select_device(0)
-#     repropa_kernel_cuda[blockspergrid, threadsperblock](P, Escat_all, G_dyad_list)
-#     cuda.close()
-#     Escat = np.sum(Escat_all, axis=0)  # sum all meshpoints' contributions
-    
-#     return Escat
