@@ -570,7 +570,8 @@ def t_sbs_HE_123_2D(geo, wavelength, selfterms, alpha, conf_dict, M):
 class DyadsQuasistatic2D123(propagators.DyadsBaseClass):
     __name__ = "3-layer environment: Quasistatic 2D '1-2-3' Green's tensors"
     
-    def __init__(self, n1=None, n2=None, n3=None, spacing=5000, ky0=None):
+    def __init__(self, n1=None, n2=None, n3=None, spacing=5000, ky0=None,
+                 ignore_incident_field_error=False):
         super().__init__()
         
         ## Dyads
@@ -607,6 +608,7 @@ class DyadsQuasistatic2D123(propagators.DyadsBaseClass):
         
         self.spacing = spacing
         self.ky0 = ky0
+        self.ignore_incident_field_error = ignore_incident_field_error
         
     def __repr__(self, verbose=False):
         """description about simulation environment defined by set of dyads"""
@@ -642,8 +644,12 @@ class DyadsQuasistatic2D123(propagators.DyadsBaseClass):
                                           fields_py.evanescent_planewave,
                                           fields_py.plane_wave,
                                           field_dipole_line_2d]:
-            raise Exception("Incompatible incident field. "+
-                            "2D Dyads are only compatible with fields `plane_wave` and `planewave`.")
+            if self.ignore_incident_field_error:
+                warnings.warn('A non-2D incident field is used! This may lead to incorrect results.')
+            else:
+                raise Exception("Incompatible incident field. "+
+                                "2D Dyads are only compatible with fields `plane_wave` and `planewave`."+
+                                "Ignore this error voia the `ignore_incident_field_error` argument.")
         
         all_inc_angles = []
         for k in efield.kwargs_permutations:
@@ -718,7 +724,7 @@ class DyadsQuasistatic2D123(propagators.DyadsBaseClass):
             else:
                 conf_dict['ky0'] = np.complex64(0)
         else:
-            raise ValueError("2D-incompatible incident field.")
+            raise ValueError("2D-incompatible incident field. Set `ky0` to override this error.")
         
         ## return a numba typed dictionary of "complex64" type,
         ## can be used to pass configuration to the green's functions
