@@ -246,6 +246,9 @@ class struct(object):
         
         epsilon = np.array(epsilon)
         self.epsilon_tensor = epsilon
+    
+    def getCenterOfMass(self):
+        return np.mean(self.geometry, axis=0)
 
 
 ## --- alias for transitional code compoatibility
@@ -1670,7 +1673,7 @@ def rect_split_ring(step, L1,L2,H,W, G=False, mesh='cube', ORIENTATION=1):
         A2 = (0<=xi<W and 0<=yi<L2 and (0 <= zi < H - 1/np.sqrt(2)))
         A3 = (L1-W<=xi<L1 and 0<=yi<L2 and (0 <= zi < H - 1/np.sqrt(2)))
         
-        if G != False:
+        if G is not False:
             A4 = ((0<=xi<(L1-G)/2 or (L1+G)/2<=xi<L1) and L2-W<=yi<L2 and (0 <= zi < H - 1/np.sqrt(2))) 
         else:
             A4 = False
@@ -2152,7 +2155,7 @@ def shift(geo, delta):
     return geo
 
 
-def center_struct(STRUCT, returnOffsets=False):
+def center_struct(STRUCT, which_axis=['x', 'y'], returnOffsets=False):
     """
     Center a structure on the x/y plane around (0,0)
     
@@ -2163,6 +2166,9 @@ def center_struct(STRUCT, returnOffsets=False):
         of the structure module; or `struct` or `simulation` object (which 
         includes the geometry information)
     
+    which_axis : list of str, default: ['x', 'y']
+        on which axis to center the structure.
+    
     returnOffsets : bool, default: False
         if True, return tuple of applied (X,Y)-offset together with structure
     
@@ -2194,28 +2200,35 @@ def center_struct(STRUCT, returnOffsets=False):
     
     DX = (np.max(X) + np.min(X)) / 2.
     DY = (np.max(Y) + np.min(Y)) / 2.
-    X -= DX
-    Y -= DY
+    DZ = (np.max(Z) + np.min(Z)) / 2.
+    if 'x' in which_axis or 'X' in which_axis:
+        X -= DX
+    if 'y' in which_axis or 'Y' in which_axis:
+        Y -= DY
+    if 'z' in which_axis or 'Z' in which_axis:
+        Z -= DZ
     
     
     if returnType == 'list':
         STRUCT = np.transpose([X,Y,Z])
         if returnOffsets:
-            return STRUCT, (DX,DY)
+            return STRUCT, (DX,DY,DZ)
         else:
             return STRUCT
     elif returnType == 'struct':
         STRUCT.geometry.T[0] = X
         STRUCT.geometry.T[1] = Y
+        STRUCT.geometry.T[2] = Z
         if returnOffsets:
-            return STRUCT, (DX,DY)
+            return STRUCT, (DX,DY,DZ)
         else:
             return STRUCT
     elif returnType == 'simulation':
         STRUCT.struct.geometry.T[0] = X
         STRUCT.struct.geometry.T[1] = Y
+        STRUCT.struct.geometry.T[2] = Z
         if returnOffsets:
-            return STRUCT, (DX,DY)
+            return STRUCT, (DX,DY,DZ)
         else:
             return STRUCT
 
